@@ -18,7 +18,7 @@ use tokio_util::io::StreamReader;
 
 use tc_error::*;
 use tc_value::Value;
-use tcgeneric::{Id, PathSegment, TCBoxTryFuture, TCTryStream};
+use tcgeneric::{Id, PathSegment, TCBoxTryFuture, TCBoxTryStream};
 
 use super::{Transaction, TxnId};
 
@@ -136,7 +136,7 @@ pub trait BlockRead<B: BlockData, F: File<B>>: Deref<Target = B> + Send {
         -> TCBoxTryFuture<<<F as File<B>>::Block as Block<B, F>>::WriteLock>;
 }
 
-pub trait BlockWrite<B: BlockData, F: File<B>>: DerefMut<Target = B> + Send {
+pub trait BlockWrite<B: BlockData, F: File<B>>: DerefMut<Target = B> + Send + Sync {
     fn downgrade(
         self,
         file: &F,
@@ -289,7 +289,7 @@ pub trait CopyFrom<D: Dir, I>: Persist<D> {
     async fn copy_from(
         instance: I,
         store: <Self as Persist<D>>::Store,
-        txn: <Self as Persist<D>>::Txn,
+        txn: &<Self as Persist<D>>::Txn,
     ) -> TCResult<Self>;
 }
 
@@ -321,7 +321,7 @@ pub trait Hash<'en, D: Dir> {
         Ok(Bytes::from(digest.to_vec()))
     }
 
-    async fn hashable(&'en self, txn: &'en Self::Txn) -> TCResult<TCTryStream<'en, Self::Item>>;
+    async fn hashable(&'en self, txn: &'en Self::Txn) -> TCResult<TCBoxTryStream<'en, Self::Item>>;
 }
 
 async fn hash_chunks<'en, T: en::IntoStream<'en> + 'en>(
